<?php

namespace App\Http\Controllers;

use App\Http\Requests\ConfirmPayment;
use App\Http\Requests\OrderAction;
use App\Http\Requests\ConfirmPaymentRequest;
use App\Models\Order;
use App\Models\PostCard;
use App\Services\Interface\OrderInterface;
use App\Services\Interface\PostCardInterface;
use App\Services\Interface\VoucherInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class OrderController extends Controller
{
    public function index(PostCardInterface $i_post_card, Request $request)
    {
        try {
            $categories = $i_post_card->getCategories();
            if (!$categories->success) return Base::error($categories->message);

            $post_cards = $i_post_card->filter(null, $request->categories, $request->search);
            if (!$post_cards->success) return Base::error($post_cards->message);

            $data = [
                'post_cards' => $post_cards->data,
                'categories' => $categories->data,
            ];

            return Base::success('', $data);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function add(OrderInterface $i_order, VoucherInterface $i_voucher, Request $request)
    {
        try {
            //! TODO: Need to move in custom validation
            $validator =  Validator::make($request->all(), [
                'card_id' => 'required',
                'quantity' => 'required',

                'bill_name' => 'required',
                // 'bill_email' => 'required',
                'bill_phone' => 'required',
                'bill_postcode' => 'required',
                'bill_city' => 'required',
                'bill_address' => 'required',
                'bill_address_type' => 'required|in:home,office,others',

                'ship_name' => 'required',
                // 'ship_email' => 'required',
                'ship_phone' => 'required',
                'ship_postcode' => 'required',
                'ship_city' => 'required',
                'ship_address' => 'required',
                'ship_address_type' => 'required|in:home,office,others',

                // 'delivery_type' => 'required|in:in_store,home_delivery',
                // 'delivery_date' => 'required',
                // 'delivery_time' => 'required',

                'payment_method' => 'required|in:card,cod,coin',
                'payment_id' => 'required',
                // 'expiry' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $post_card = PostCard::find($request->card_id);
            if (!$post_card) return Base::error("Post Card not found"); // DONE

            $inputs = $request->all();
            $inputs['user_id'] = Auth::id();
            $inputs['price'] = $post_card->price;

            if (isset($request->discount_code)) {
                $voucher_request = [
                    "voucher_code" => $request->discount_code,
                    "shop_id" => $post_card->shop_id,
                    "user_id" => Auth::id(),
                ];

                $voucher_response = $i_voucher->checkVoucher($voucher_request);
                if (!$voucher_response->success) return Base::error($voucher_response->message);
                $voucher_data = $voucher_response->data;
                $inputs['discount'] = $voucher_data->amount;
                $inputs['discount_code'] = $voucher_data->voucher_code;
            }

            $res = $i_order->add($inputs);
            if (!$res->success) return Base::error($res->message);

            return Base::success('Order Submitted', $res->data);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function confirmPayment(OrderInterface $i_order, ConfirmPaymentRequest $request)
    {

        $res = $i_order->confirmPayment($request);
        if (!$res->success) return Base::error($res->message);
        return Base::success('Order Submitted', $res->data);
    }

    public function addFeedback(OrderInterface $i_order, PostCardInterface $i_post_card, Request $request)
    {
        try {
            $validator =  Validator::make($request->all(), [
                'order_id' => 'required',
                'rate' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $order = Order::find($request->order_id);
            if (!$order) return Base::error("Order not found"); // DONE

            if ($order->user_id !== Auth::id()) return Base::error("You are not authorized to rate this order"); // DONE

            if ($order->status  != 'completed') return Base::error("This order is not completed yet");  // pending,processing,completed,canceled,rejected //DONE
            if ($order->rate != null) return Base::error("Already rated to this order"); // DONE
            $res = $i_order->addFeedback($order, $request->rate, $request->review);
            if (!$res->success) return Base::error($res->message);


            $res2 = $i_post_card->updateRate($order->card_id);
            if (!$res2->success) return Base::error($res2->message);

            return Base::success('Feedback added', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function orderHistory(OrderInterface $i_order, Request $request)
    {
        try {
            $res = $i_order->orderHistory(Auth::id());
            if (!$res->success) return Base::error($res->message);

            return Base::success('Order List Found', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function orderHistoryShow(OrderInterface $i_order, Request $request)
    {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $res = $i_order->orderHistoryShow($request);
            if (!$res->success) return Base::error($res->message);

            return Base::success('Order List Found', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    //* Customer order filter lis

    public function orderHistoryFilter(OrderInterface $orderInterface, Request $request)
    {
        try {
            $data = $orderInterface->orderHistoryFilter($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::error($e);
        }
    }

    public function orderAction(OrderInterface $orderInterface, OrderAction $request)
    {
        $data = $orderInterface->orderAction($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }


    //* TEST function //* 
    //! don't need to review it.

    public function allPaymentIntent(Request $request)
    {


        $secret_key = config('app.stripe_key');
        $stripe = new \Stripe\StripeClient($secret_key);
        $data = $stripe->paymentIntents->all();

        return $data;
    }

    // for react application stripe payment intent
    public function createPaymentIntent(Request $request)
    {
        $validator =  Validator::make($request->all(), ['amount' => 'required']);
        if ($validator->fails()) return Base::validation($validator);

        try {
            // $secret_key = 'sk_test_51KLNAfBiwDSxURARosEbsc5hi4zajIpDiWBzdhHJZ9YyHPzPoqd1sR2ergNAmSWmEUC9gYIK61p8S4OkH5ajpfGM00yIXzcFSc';
            $secret_key = config('app.stripe_key');
            $stripe = new \Stripe\StripeClient($secret_key);

            if (isset($request->payment_id)) {
                $data = $stripe->paymentIntents->retrieve($request->payment_id);
            } else {
                $data = $stripe->paymentIntents->create([
                    'amount' => $request->amount,
                    'currency' => 'jpy',
                    'payment_method_types' => ['card'],
                ]);
            }


            $output = [
                'clientSecret' => $data->client_secret,
                'payment_id' => $data->id,
            ];

            // echo json_encode($output);
            return Base::pass("Payment Intent Information", $output);
            // return $output;
        } catch (Exception $e) {
            // http_response_code(500);
            // echo json_encode(['error' => $e->getMessage()]);
            // return $e->getMessage();
            return Base::exception_fail($e);
        }


        return $data;
    }
}
