<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Http\Resources\CategoryResource;
use App\Models\Category;
use App\Models\Contact;
use App\Models\Order;
use App\Models\RecommendedCategory;
use App\Models\User;
use App\Services\Interface\CategoryInterface;
use App\Traits\GlobalTrait;
use Exception;

class CategoryRepository implements CategoryInterface
{

    use GlobalTrait;

    public function index()
    {
        try {
            // return $order = Order::groupBy('card_id')->get();
            $category = Category::withCount('cards as total_cards')
                ->with('cards', function ($q) {
                    $q->withCount('orders as total_orders')
                        ->withCount('pendingOrders as total_pending_orders');
                })
                ->with('recommended')
                ->get();

            $item = $category->map(function ($k) {
                return [
                    "id" => $k->id,
                    "name" => $k->name,
                    "image" => $k->image,
                    "is_active" => $k->is_active,
                    "total_cards" => $k->total_cards,
                    "total_orders" => $k->cards->sum('total_orders'),
                    "total_pending_orders" => $k->cards->sum('total_pending_orders'),
                    "is_recommended" => $k->recommended != null ? $k->recommended->is_active : 0,
                ];
            });


            return Base::pass("All category list fetched", $item); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function add($request)
    {
        try {
            $new_category  = new Category();
            $new_category->name = $request->name;
            if (isset($request->image)) {
                $new_category->image = GlobalTrait::imageUpload($request->image, 'category');
            }

            $new_category->save();
            return Base::pass("New Category added"); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function update($request)
    {
        try {
            $category  = Category::find($request->id);
            if (!$category) return Base::fail("Category does not found");
            if (isset($request->name)) {
                $category->name = $request->name;
            }

            if (isset($request->image)) {

                if ($category->image) {
                    if (file_exists(public_path() . $category->image)) {
                        unlink(public_path() . '/' . $category->image);
                    }
                }
                $category->image = GlobalTrait::imageUpload($request->image, 'category');
            }

            $category->save();
            return Base::pass("Category updated");
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function toggle($request)
    {
        try {
            $category  = Category::find($request->id);
            if (!$category) return Base::fail("Category does not found");

            $category->is_active = $category->is_active ? false : true;

            $category->save();
            return Base::pass("Category status toggled");
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function delete($request)
    {
        try {
            $category  = Category::find($request->id);
            if (!$category) return Base::fail("Category does not found");

            // $category->is_active = $category->is_active ? false : true;
            if ($category->image) {
                if (is_file(public_path() . '/' . $category->image)) {
                    unlink(public_path() . '/' . $category->image);
                }
            }
            $category->delete();
            return Base::pass("Category has been trashed");
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function getActiveCategory()
    {
        try {
            $category = Category::active()->withCount('activeCards as card_count')->get();

            return Base::pass("All category list fetched", $category);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function addRecommended($data)
    {
        try {
            $recommended = RecommendedCategory::where('category_id', $data->category_id)->first();

            if ($recommended) {

                if ($recommended->is_active == 1) $recommended->is_active = 0;
                else $recommended->is_active = 1;
                $recommended->save();
            } else {
                $new_recommended = new RecommendedCategory();

                $new_recommended->category_id = $data->category_id;

                $new_recommended->save();
            }


            return Base::pass("New recommended category updated");
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function getRecommendedCategoryList()
    {
        try {
            $recommended = RecommendedCategory::where('is_active', true)->get();
            $data = [];
            foreach ($recommended as $item) {
                $value = [
                    "category_id" => $item->category_id,
                    "category_name" => $item->category->name ?? "NA",
                ];
                array_push($data, $value);
            }
            return Base::pass("Recommended category list", $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
