<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Http\Requests\TogglePushOrEmailNotificationRequest;
use App\Http\Resources\UserResource;
use App\Jobs\SendOtpJob;
use App\Models\Shop;
use App\Services\Interface\UserInterface;
use App\Models\User;
use App\Models\UserInfo;
use App\Traits\GlobalTrait;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Laravel\Socialite\Facades\Socialite;

class UserRepository implements UserInterface
{

    use GlobalTrait;

    public function login($credentials, $role)
    {
        try {
            // ! Checking Credentials
            if (!Auth::attempt($credentials)) return Base::fail('Invalid Credentials'); // DONE
            // ! Checking app role
            if (Auth::user()->role != $role) return Base::fail("You can not login this app with this account"); // DONE

            $accessToken = Auth::user()->createToken('authToken')->accessToken;

            $data = [
                'token' => $accessToken,
                'user' => new UserResource(Auth::user()),
            ];

            // ! Checking if verified
            if (Auth::user()->is_verified) {
                return Base::pass('Login Successful', $data);
            } else {
                $is_otp_sent = $this->sendOTP(Auth::user());
                if (!$is_otp_sent) return Base::error('Login Failed, Please try again'); // DONE

                return Base::pass('Logged in successful, Please verify your account', $data); // DONE
            }
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function socialLogin($request)
    {
        try {
            $is_google_provider =  $request['provider'] === 'google' ? true : false;
            // if ($is_google_provider) {
            //     $google_client_id = config('app.google_client_id');
            //     $client = new \Google_Client(['client_id' => $google_client_id]);
            //     $social_user = $client->verifyIdToken($request['token']);
            //     if($social_user == false) return Base::fail("Can not authenticated user");
            // } else 
            $social_user = Socialite::driver($request['provider'])->userFromToken($request['token']);

            // if(!$is_google_provider) $data = response()->json($social_user);
            // if(!$is_google_provider) 
            $data = response()->json($social_user);

            $user_name = $data->getData()->name;
            $user_email =  $data->getData()->email;
            $user_id =  $data->getData()->id;
            $user_image =  $data->getData()->avatar;
            // $user_name = $is_google_provider? $social_user['name'] : $data->getData()->name;
            // $user_email = $is_google_provider? $social_user['email'] : $data->getData()->email;
            // $user_id = $is_google_provider? $social_user['sub'] : $data->getData()->id;
            // $user_image = $is_google_provider? $social_user['picture'] : $data->getData()->avatar;

            $role = $request['role'];

            $user = User::where(["login_id" => $user_id, "login_type" => $request['provider']])->first();

            if ($user != null) {
                $accessToken = $user->createToken('authToken')->accessToken;
                $user_data = new UserResource($user);
            } else {
                if ($user_email != null) {

                    $user_find = User::where("email", $user_email)->first();
                    if ($user_find) return Base::error("User already exist with this email.");
                    else {
                        $new_user = new User();

                        $new_user->name = $user_name;
                        $new_user->email = $user_email ?? null;
                        $new_user->login_id = $user_id;
                        $new_user->image_url = $user_image;
                        $new_user->login_type = $request['provider'];
                        $new_user->is_active = true;
                        $new_user->is_verified = true;
                        $new_user->role = $role;
                        $new_user->refer_code = $this->generateReferCode();
                        $new_user->save();

                        $accessToken = $new_user->createToken('authToken')->accessToken;

                        $user_data = new UserResource($new_user);
                    }
                } else {
                    $new_user = new User();

                    $new_user->name = $user_name;
                    $new_user->email = $user_email ?? null;
                    $new_user->login_id = $user_id;
                    $new_user->image_url = $user_image;
                    $new_user->login_type = $request['provider'];
                    $new_user->is_active = true;
                    $new_user->is_verified = true;
                    $new_user->role = $role;
                    $new_user->refer_code = $this->generateReferCode();
                    $new_user->save();

                    $accessToken = $new_user->createToken('authToken')->accessToken;

                    $user_data = new UserResource($new_user);
                }
            }
            $data1 = [
                'token' => $accessToken,
                'user' => $user_data,
            ];
            return Base::pass('Login Successful', $data1);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function generateReferCode()
    {
        $str_result = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        $random = substr(str_shuffle($str_result), 0, 6);

        $found_duplicate = User::where('refer_code', $random)->first();
        if ($found_duplicate) $this->generateReferCode();

        return $random;
    }


    public function createUser($data)
    {
        try {
            $user = new User();
            $user->name = $data['name'];
            $user->email = strtolower($data['email']);
            $user->password = Hash::make($data['password']);
            $user->role = $data['role'];
            $user->refer_code = $this->generateReferCode();
            $user->refer_by = $data['refer_code'] ?? null;
            $user->save();

            return Base::pass(null, $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }



    public function createShop($user_id)
    {
        try {
            $shop = new Shop();
            $shop->user_id = $user_id;
            $shop->save();

            return Base::pass(null, $shop);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function sendOtp($user)
    {
        try {
            $generateOTP = rand(1000, 9999);
            $user->otp = Hash::make($generateOTP);
            $user->otp_created_at = Base::now();
            $user->save();

            $details = [
                'greeting' => 'Hi ' . $user->name,
                'body' => 'Here is your verification code that\'s valid for only 5 Mins. If its not you please ignore it.',
                'otp' => 'Verification Code: ' . $generateOTP,
            ];
            // $user->notify(new VerifyOTP($details));
            dispatch(new SendOtpJob($user, $details));

            return Base::pass(null, $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function getUserProfile()
    {
        try {
            $data = new UserResource(Auth::user());

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function updateUser($user, $data)
    {
        try {
            if (isset($data['image_url'])) {
                $img = GlobalTrait::imageUpload($data['image_url'], 'users');
                $user->image_url = $img;
            }

            if (isset($data['name'])) $user->name = $data['name'];
            if (isset($data['email'])) $user->email = $data['email'];
            if (isset($data['phone'])) $user->phone = $data['phone'];
            if (isset($data['address'])) $user->address = $data['address'];
            if (isset($data['post_code'])) $user->post_code = $data['post_code'];
            if (isset($data['city'])) $user->city = $data['city'];

            $user->save();

            return Base::pass("User profile updated", $user); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    public function updatePassword($user, $password)
    {
        try {
            $user->password = Hash::make($password);
            $user->save();
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminCustomerChangePassword(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'password' => 'required|confirmed|min:6',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = User::find($request['user_id']);

            if (!$user) return Base::fail("User not found");

            $user->password = Hash::make($request['password']);
            $user->save();
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function updateLanguage($user, $code)
    {
        try {
            $user->lang_code = $code;
            $user->save();
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function togglePushNotification($user)
    {
        try {
            $user->is_push_notification = !$user->is_push_notification;
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function toggleEmailNotification($user)
    {
        try {
            $user->is_email_notification = !$user->is_email_notification;
            $data = new UserResource($user);

            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function togglePushOrEmailNotification(TogglePushOrEmailNotificationRequest $request)
    {
        try {
            $user = Auth::user();

            if ($request->type == 'push_notification') $user->is_push_notification = !$user->is_push_notification;
            else  $user->is_email_notification = !$user->is_email_notification;

            $user->save();

            $data = [
                'is_push_notification' => $user->is_push_notification == 1 ? 1 : 0,
                'is_email_notification' => $user->is_email_notification == 1 ? 1 : 0,
            ];

            return Base::success('Settings Changed', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function delete($user)
    {
        try {
            // $user->delete();
            $user->name = "Gifty User";
            $user->email = "Gifty User" . $user->id;
            $user->password = "Gifty User";
            $user->login_id = null;
            $user->image = null;
            $user->image_url = null;
            $user->phone = null;
            $user->address = null;
            $user->is_verified = false;
            $user->is_active = false;
            $user->is_push_notification = false;
            $user->is_email_notification = false;
            $user->refer_by = null;
            $user->refer_code = null;
            $user->city = null;
            $user->post_code = null;
            $user->save();

            return Base::pass(null, $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function deleteText()
    {
        try {
            //Code will be here
            // if (Auth::user()->role == 'customer') {
            //     $data = "<h2>Delete profile message for customer</h2>";
            // } else {
            //     $data = "<h2>Delete profile message for shop</h2>";
            // }


            if (Auth::user()->role == 'customer') {
                if (Auth::user()->lang_code == 'en') {
                    $data = "<p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>Our app preserves no user data after deleting an account. So if you want to delete your account you may first check all the orders on pending and other records. All of the pending orders will be cancelled after you delete your account.&nbsp;</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>What will be removed?</span></p><p><br></p><p><br></p><p><br></p><ul><li>The account info that you have provided when you registered and all the related data will be removed.</li></ul><p><br></p><ul><li>All the saved addresses will be removed.</li></ul><p><br></p><ul><li>All the favorite cards will be removed.</li></ul><p><br></p><ul><li>Order history will be removed and all of the pending orders will be cancelled.</li></ul><p><br></p><ul><li>We will be not responsible upon refunding any money once you delete your account.</li></ul><p><br></p><ul><li>Your account data will be temporarily stored in our database upto 30 days. After that all the related information of your account will be permanently deleted.</li></ul><p><br></p><p><br></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>So please proceed with cautions.&nbsp;</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>Finally, are you sure you want to delete your account? If “Yes” then proceed with agreeing all the condition mentioned above. If the answer is 'No' , then please go back to other related pages of the app. &nbsp;</span></p>";
                } else {
                    $data = "<p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>当社のアプリは、アカウントを削除した後、ユーザーデータを保持しません。そのため、アカウントを削除したい場合は、まず保留中の注文やその他の記録をすべて確認することができます。アカウントを削除すると、保留中の注文はすべてキャンセルされます。</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>何が削除されるのですか？</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>1. 登録時に入力されたアカウント情報および関連するすべてのデータが削除されます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>2. 2.保存されているすべてのアドレスが削除されます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>3. 3.お気に入りカードがすべて削除されます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>4. 注文履歴が削除され、保留中の注文がすべてキャンセルされます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>5. 5.アカウントを削除された場合、当社は返金に応じられません。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>6. 6.お客様のアカウントデータは、最大30日間、当社のデータベースに一時的に保存されます。その後、あなたのアカウントのすべての関連情報は、永久に削除されます。</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>そのため、注意深く手続きを行ってください。</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>最後に、本当にアカウントを削除しますか？もし「はい」なら、上記のすべての条件に同意して進んでください。いいえ」の場合は、アプリの他の関連ページにお戻りください。</span></p>";
                }
            } else {
                if (Auth::user()->lang_code == 'en') {
                    $data = "<p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>Our app preserves no user data after deleting an account. So if you want to delete your account you may first check all the orders on pending and other records. All of the pending orders will be cancelled after you delete your account.&nbsp;</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>What will be removed?</span></p><p><br></p><p><br></p><p><br></p><ul><li>The account info that you have provided when you registered and all the related data will be removed.</li></ul><p><br></p><ul><li>All the saved addresses will be removed.</li></ul><p><br></p><ul><li>All the favorite cards will be removed.</li></ul><p><br></p><ul><li>Order history will be removed and all of the pending orders will be cancelled.</li></ul><p><br></p><ul><li>We will be not responsible upon refunding any money once you delete your account.</li></ul><p><br></p><ul><li>Your account data will be temporarily stored in our database upto 30 days. After that all the related information of your account will be permanently deleted.</li></ul><p><br></p><p><br></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>So please proceed with cautions.&nbsp;</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>Finally, are you sure you want to delete your account? If “Yes” then proceed with agreeing all the condition mentioned above. If the answer is 'No' , then please go back to other related pages of the app. &nbsp;</span></p>";
                } else {
                    $data = "<p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>当社のアプリは、アカウントを削除した後、ユーザーデータを保持しません。そのため、アカウントを削除したい場合は、まず保留中の注文やその他の記録をすべて確認することができます。アカウントを削除すると、保留中の注文はすべてキャンセルされます。</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>何が削除されるのですか？</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>1. 登録時に入力されたアカウント情報および関連するすべてのデータが削除されます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>2. 2.保存されているすべてのアドレスが削除されます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>3. 3.お気に入りカードがすべて削除されます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>4. 注文履歴が削除され、保留中の注文がすべてキャンセルされます。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>5. 5.アカウントを削除された場合、当社は返金に応じられません。</span></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>6. 6.お客様のアカウントデータは、最大30日間、当社のデータベースに一時的に保存されます。その後、あなたのアカウントのすべての関連情報は、永久に削除されます。</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>そのため、注意深く手続きを行ってください。</span></p><p><br></p><p><span style='background-color: rgba(32, 33, 36, 0.04); color: rgb(32, 33, 36);'>最後に、本当にアカウントを削除しますか？もし「はい」なら、上記のすべての条件に同意して進んでください。いいえ」の場合は、アプリの他の関連ページにお戻りください。</span></p>";
                }
            }


            return Base::pass(null, $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }


    //* Admin

    public function adminCustomerIndex($request)
    {
        try {
            //Code will be here
            $take = isset($request['take']) ? $request['take'] : 20;
            $customer = User::customer()
                ->withCount('customerOrders as total_orders')
                ->with('customerLatestOrders', function ($q) {
                    // return $q->pivot(['card_name']);
                })
                ->paginate($take);
            // ->makeVisible([
            //     'is_verified',
            //     'is_active',
            // ]);
            return Base::pass("All Customer list", $customer); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminCustomerFilter($request)
    {
        try {
            //Code will be here
            $take = isset($request['take']) ? $request['take'] : 20;
            $search = isset($request['search']) ? $request['search'] : null;
            $customer = User::customer()
                ->where('name', 'like', '%' . $search . '%')
                ->withCount('customerOrders as total_orders')
                ->with('customerLatestOrders', function ($q) {
                    // return $q->pivot(['card_name']);
                })
                ->paginate($take);
            // ->makeVisible([
            //     'is_verified',
            //     'is_active',
            // ]);
            return Base::pass("All Customer list", $customer); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminCustomerShow($data)
    {
        try {
            //Code will be here
            $customer = User::customer()
                ->where('id', $data['id'])
                ->with(['userInfo'])
                ->first();
            return Base::pass("Customer details", $customer); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminStatusToggle($data)
    {
        try {
            //Code will be here
            $user = User::find($data['user_id']);
            $user->is_active = !$user->is_active;
            $user->save();
            $user->refresh();
            $data = new UserResource($user);

            return Base::pass("User status changed", $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function adminVidStatusToggle($data)
    {
        try {
            //Code will be here
            $user = User::find($data['user_id']);
            // $user->is_active = !$user->is_active;
            // $user->save();
            $user_info = $user->userInfo;
            if ($user_info == null) return Base::fail('Verification information not updated');
            // if()
            $user_info->status = $data['status'];
            $user_info->is_vid_verified = $data['status'] == 'verified' ? true : false;
            $user_info->save();
            $user->refresh();
            $data = new UserResource($user);

            //!TODO: Send notification to user if verified.

            return Base::pass("User verification information updated", $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function adminUserUpdate($request)
    {
        try {
            $user = User::find($request['user_id']);
            if (!$user) return Base::error("User not found"); // DONE

            $data = $this->updateUser($user, $request);
            return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function adminAddCustomer($request)
    {
        try {
            // $user = User::find($request['user_id']);
            // if(!$user) return Base::error("User not found.");
            $request['role'] = 'customer';
            $data = $this->createUser($request);
            if ($data->success == true) {
                $user = $data->data;
                if (isset($request['image_url'])) {
                    $img = GlobalTrait::imageUpload($request['image_url'], 'users');
                    $user->image_url = $img;
                }
                $user->is_verified = true;
                $user->phone = isset($request['phone']) ? $request['phone'] : null;
                $user->address = isset($request['address']) ? $request['address'] : null;
                $user->save();
            }
            return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function vIDVerify($request)
    {
        try {
            $user = Auth::user();

            $user_info = $user->userInfo;

            if ($user_info == null) $user_info = new UserInfo();

            $user_info->user_id = $user->id;
            $user_info->vid_front = GlobalTrait::imageUploadID($request['vid_front'], 'users', $user->id);
            $user_info->vid_back = GlobalTrait::imageUploadID($request['vid_back'], 'users', $user->id);
            $user_info->is_vid_verified = false;
            $user_info->status = 'pending';
            $user_info->type = $request['type'];
            $user_info->save();

            return Base::pass('User verification id uploaded successfully');
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function vIDVerifyInfo($request)
    {
        try {
            // $auth = Auth::user()->id;

            $user = UserInfo::where('user_id', Auth::user()->id)->first();

            return Base::pass('User verification info', $user);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
